/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.internal;

import java.util.jar.Attributes;
import java.util.jar.JarFile;
import java.util.jar.Manifest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * {@link GuardVersion} - Extracting Version information from the Manifest file
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v2.4.0
 */
public class GuardVersion {

    private static final Logger logger = LoggerFactory.getLogger(GuardVersion.class);
    private static final String UNKNOWN_VERSION_STRING = "n/a";
    private static final String MANIFEST_VERSION = "Version";
    private static final String MANIFEST_REVISION = "Revision";
    private static final String MANIFEST_BUILD_DATE = "BuildDate";

    private static String guardVersion;

    /**
     * Internal method to construct the version string from the manifest attributes
     *
     * @param manifestAttributes the manifest attributes
     * @return the version string
     */
    private static String createVersionString(Attributes manifestAttributes) {
        return manifestAttributes.getValue(MANIFEST_VERSION) +
            "-rev" +
            manifestAttributes.getValue(MANIFEST_REVISION) +
            ":" +
            manifestAttributes.getValue(MANIFEST_BUILD_DATE);
    }

    /**
     * Loads the guard version from the manifest file
     */
    public static void loadGuardVersion() {
        try {
            String location = GuardVersion.class.getProtectionDomain().getCodeSource().getLocation().toString();
            location = location.replace("file:", "");
            if (location.contains(".jar")) {
                try (JarFile jarFile = new JarFile(location)) {
                    Manifest manifest = jarFile.getManifest();
                    if (manifest == null) {
                        guardVersion = "unknown";
                    } else {
                        Attributes attributes = manifest.getMainAttributes();
                        String version = createVersionString(attributes);
                        guardVersion = version;
                        logger.info("OX Guard Version: " + guardVersion);
                    }
                }
            } else {
                guardVersion = null;
            }
        } catch (Exception ex) {
            logger.error("Problem getting OX Guard version ", ex);
        }
    }

    /**
     * @return the guard version defined in the bundle's manifest file
     */
    public static String getGuardVersion() {
        return (guardVersion == null || guardVersion.isEmpty()) ? UNKNOWN_VERSION_STRING : guardVersion;
    }
}
