/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.servlets.keymanagement.responses;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import org.bouncycastle.bcpg.attr.ImageAttribute;
import org.bouncycastle.openpgp.PGPPublicKey;
import org.bouncycastle.openpgp.PGPUserAttributeSubpacketVector;
import com.openexchange.pgp.keys.tools.PGPKeysUtil;

/**
 * {@link PublicKeyInfoResponse}
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v2.6.0
 */
public class PublicKeyInfoResponse {

    private final PGPPublicKey key;
    private final boolean hasPrivateaKey;
    private final boolean usedForEncryption;

    /**
     * Initializes a new {@link PublicKeyInfoResponse}.
     * @param key
     * @param usedForEncryption Whether or not Guard is considering using this key for encryption.
     */
    public PublicKeyInfoResponse(PGPPublicKey key, boolean usedForEncryption) {
        this(key, false, usedForEncryption);
    }

    /**
     * Initializes a new {@link PublicKeyInfoResponse}.
     * @param key
     * @param hasPrivateaKey Whether or not this key has a corresponding private key
     * @param usedForEncryption Whether or not Guard is considering using this key for encryption.
     */
    public PublicKeyInfoResponse(PGPPublicKey key, boolean hasPrivateaKey, boolean usedForEncryption) {
        this.key = key;
        this.hasPrivateaKey = hasPrivateaKey;
        this.usedForEncryption = usedForEncryption;
    }

    public String getId() {
        return Long.toString(this.key.getKeyID());
    }

    public boolean isMasterKey() {
        return this.key.isMasterKey();
    }

    public String getFingerPrint() {
        return PGPKeysUtil.getFingerPrint(this.key.getFingerprint());
    }

    public Date getCreationTime () {
        return key.getCreationTime();
    }

    public long getValidSeconds() {
        return key.getValidSeconds();
    }

    public String[] getUserIds() {
        ArrayList<String> userIds = new ArrayList<String>();
        Iterator iter = key.getUserIDs();
        while(iter.hasNext()) {
            userIds.add((String)iter.next());
        }
        return userIds.toArray(new String[userIds.size()]);
    }

    public boolean isEncryptionKey() {
        return PGPKeysUtil.isEncryptionKey(key);
    }

    public boolean isUsedForEncryption() {
       return this.usedForEncryption;
    }

    public boolean isRevoked() {
        return key.hasRevocation();
    }

    public boolean isExpired() {
        return PGPKeysUtil.isExpired(key);
    }

    public boolean getHasPrivateKey() {
        return hasPrivateaKey;
    }

    public PublicKeyImageResponse[] getImages() {
        ArrayList<PublicKeyImageResponse> ret = new ArrayList<PublicKeyImageResponse>();
        Iterator<PGPUserAttributeSubpacketVector> userAttributes = this.key.getUserAttributes();
        while (userAttributes.hasNext()) {
            ImageAttribute imageAttribute = userAttributes.next().getImageAttribute();
            if (imageAttribute != null && imageAttribute.getType() == 1) {
                ret.add(new PublicKeyImageResponse(imageAttribute));
            }
        }
        return ret.toArray(new PublicKeyImageResponse[ret.size()]);
    }
}